#pragma once

#include "Abstract.hpp"
#include "../data/math.hpp"

namespace contouring {
    /// Generating mesh for chunks 1:1
    class AbstractFlat: public Abstract {
    public:
        AbstractFlat(const std::string &str);
        virtual ~AbstractFlat() {}

        /// Display ImGui config
        void onGui() override;

        std::string getOptions() const override;
        std::pair<float, float> getFarRange() const override;

        /// Get buffers in frustum with model matrices
        /// @note buffers invalidated after update
        void getModels(std::vector<std::pair<glm::mat4, buffer::Abstract *const>> &out, const std::optional<geometry::Frustum> &frustum, const glm::llvec3& offset, int density) override;

    protected:
        size_t clear(const voxel_pos &, const world::area_map &areas);

        robin_hood::unordered_map<area_id, robin_hood::pair<area_pos, robin_hood::unordered_map<chunk_pos, buffer::Abstract *>>> buffers;

        int loadDistance = 3;
        int keepDistance = 4;
    };
}