#include "surrounding.hpp"

#include "../world/Area.hpp"
#include "../data/math.hpp"

using namespace geometry;
namespace contouring::surrounding {
    bool load(faces &out, const chunk_pos &chunkPos, const world::ChunkContainer &chunks) {
        {
            const auto chunk = chunks.findInRange(chunkPos);
            if (!chunk.has_value())
                return false;

            out[CENTER] = chunk.value();
        }
        for (size_t i = 0; i < CENTER; i++) {
            const auto chunk = chunks.findOrEmpty(chunkPos + g_face_offsets[i]);
            if (!chunk.has_value())
                return false;

            out[i] = chunk.value();
        }
        return true;
    }

    std::pair<glm::idx, glm::idx> getNeighborIdx(glm::idx idx, Face face) {
        switch (face) {
        case Face::Forward:
            if (idx % glm::IDX_LENGTH >= glm::IDX_LENGTH - 1)
                return {static_cast<int>(Face::Forward), idx - (glm::IDX_LENGTH - 1)};
            return {CENTER, idx + 1};

        case Face::Backward:
            if (idx % glm::IDX_LENGTH <= 0)
                return {static_cast<int>(Face::Backward), idx + (glm::IDX_LENGTH - 1)};
            return {CENTER, idx - 1};

        case Face::Up:
            if ((idx / glm::IDX_LENGTH) % glm::IDX_LENGTH >= glm::IDX_LENGTH - 1)
                return {static_cast<int>(Face::Up), idx - (glm::IDX_LENGTH2 - glm::IDX_LENGTH)};
            return {CENTER, idx + glm::IDX_LENGTH};

        case Face::Down:
            if ((idx / glm::IDX_LENGTH) % glm::IDX_LENGTH <= 0)
                return {static_cast<int>(Face::Down), idx + (glm::IDX_LENGTH2 - glm::IDX_LENGTH)};
            return {CENTER, idx - glm::IDX_LENGTH};

        case Face::Right:
            if (idx / glm::IDX_LENGTH2 >= glm::IDX_LENGTH - 1)
                return {static_cast<int>(Face::Right), idx - (glm::IDX_SIZE - glm::IDX_LENGTH2)};
            return {CENTER, idx + glm::IDX_LENGTH2};

        case Face::Left:
            if (idx / glm::IDX_LENGTH2 <= 0)
                return {static_cast<int>(Face::Left), idx + (glm::IDX_SIZE - glm::IDX_LENGTH2)};
            return {CENTER, idx - glm::IDX_LENGTH2};

        default:
            return {CENTER, idx};
        }
    }

    bool load(corners &out, const chunk_pos &chunkPos, const world::ChunkContainer &chunks) {
        {
            const auto chunk = chunks.findInRange(chunkPos);
            if(!chunk.has_value())
                return false;

            out[0] = chunk.value();
        }
        for (size_t i = 1; i < 8; i++) {
            const auto chunk = chunks.findOrEmpty(chunkPos + g_corner_offsets[i]);
            if (!chunk.has_value())
                return false;

            out[i] = chunk.value();
        }
        return true;
    }
}
