#include "Camera.hpp"

#include <glm/gtc/matrix_transform.hpp>
#include "../render/window.hpp"

Camera::Camera(GLFWwindow *window, const InputMap& inputs, const Camera::options& opt): window(window), inputs(inputs),
		Position(voxel_pos(0), 1),  o(opt){
	updateProjection();
}
Camera::~Camera() { }

void Camera::updateProjection() {
	ProjectionMatrix = glm::perspective(o.fov, RATIO, o.near, o.far);
}

void Camera::update(bool captureMouse, bool captureKeys, float deltaTime) {
	// Get mouse position
	if(captureMouse) {
		int viewportX, viewportY;
		glfwGetWindowSize(window, &viewportX, &viewportY);
		if(capturingMouse) {
			double xPos, yPos;
			glfwGetCursorPos(window, &xPos, &yPos);

			// Compute new orientation
			HorizontalAngle += o.sensibility * 0.0001f * float(viewportX / 2 - xPos);
			VerticalAngle += o.sensibility * 0.0001f * float(viewportY / 2 - yPos);
		} else {
			glfwSetInputMode(window, GLFW_CURSOR, GLFW_CURSOR_DISABLED);
			capturingMouse = true;
		}
		// Reset mouse position for next frame
		glfwSetCursorPos(window, viewportX / 2, viewportY / 2);
	} else if (capturingMouse) {
		glfwSetInputMode(window, GLFW_CURSOR, GLFW_CURSOR_NORMAL);
		capturingMouse = false;
	}

	// Direction : Spherical coordinates to Cartesian coordinates conversion
	glm::vec3 direction = getDirection();

	// Right vector
	glm::vec3 right = glm::vec3(
		sin(HorizontalAngle - 3.14f / 2.0f),
		0,
		cos(HorizontalAngle - 3.14f / 2.0f));

	// Up vector
	glm::vec3 up = glm::cross(right, direction);

	if(captureKeys) {
		// Move forward
		if (inputs.isDown(Input::Forward)) {
			Position.offset += direction * deltaTime * o.speed;
		}
		// Move backward
		if (inputs.isDown(Input::Backward)) {
			Position.offset -= direction * deltaTime * o.speed;
		}
		// Strafe right
		if (inputs.isDown(Input::Right)) {
			Position.offset += right * deltaTime * o.speed;
		}
		// Strafe left
		if (inputs.isDown(Input::Left)) {
			Position.offset -= right * deltaTime * o.speed;
		}
		// Move up
		if (inputs.isDown(Input::Up)) {
			Position.offset += up * deltaTime * o.speed;
		}
		// Move down
		if (inputs.isDown(Input::Down)) {
			Position.offset -= up * deltaTime * o.speed;
		}
		Position.center();
	}

	// MAYBE: only if moved
	// MAYBE: save frustum
	// Camera matrix
	ViewMatrix = glm::lookAt(
		Position.offset,			  // Camera is here
		Position.offset + direction, // and looks here : at the same position, plus "direction"
		up					  // Head is up (set to 0,-1,0 to look upside-down)
	);
}