#pragma once

#include "InputMap.hpp"

#include <GL/glew.h>
#include <GLFW/glfw3.h>
#include "../data/glm.hpp"
#include "../data/geometry/Frustum.hpp"
#include "../data/geometry/Ray.hpp"
#include "../world/position.h"

/// Moving perspective camera
class Camera {
public:
    struct options {
        float fov = glm::radians(70.f);
        float near = 0.1;
        float far = 64;

        float speed = 5.0f;
        int sensibility = 50;
    };

    Camera(GLFWwindow*, const InputMap&, const options&);
    ~Camera();

    void update(bool captureMouse, bool captureKeys, float deltaTime);
    void setOptions(const options &options) {
        o = options;
        updateProjection();
    }

    glm::vec3 getDirection() const { return glm::vec3(cos(VerticalAngle) * sin(HorizontalAngle), sin(VerticalAngle), cos(VerticalAngle) * cos(HorizontalAngle)); }
    inline geometry::Frustum getFrustum() const { return geometry::Frustum(ViewMatrix, ProjectionMatrix); }
    inline geometry::Ray getRay() const { return geometry::Ray(Position, getDirection(), o.far); }

    constexpr glm::mat4 getViewMatrix() const { return ViewMatrix; }
    constexpr glm::mat4 getProjectionMatrix() const { return ProjectionMatrix; }
    camera_pos getPosition() const { return Position; }
    constexpr float getDepth() const { return o.far; }

private:
    GLFWwindow *window;
    const InputMap &inputs;

    glm::mat4 ViewMatrix;
    glm::mat4 ProjectionMatrix;
    void updateProjection();

    camera_pos Position;

    float HorizontalAngle = 3.14f;
    float VerticalAngle = 0.0f;

    bool capturingMouse = false;
    options o;
};
