#pragma once

#include <GL/glew.h>
#include <glm/glm.hpp>
#include <vector>

#include "Abstract.hpp"
#include "VertexData.hpp"

namespace buffer {
    /// OpenGL VertexBuffer with IndexBuffer
    class ShortIndexed: public Abstract {
    public:
        /// Preindexed buffer data
        struct Data {
            std::vector<GLushort> indices;
            std::vector<glm::vec3> vertices;
            std::vector<GLushort> materials;
            std::vector<glm::vec3> normals;

            Data() { }
            Data(const std::vector<VertexData> &vertices, const std::vector<GLushort> &indices);
            Data(const std::vector<VertexData> &vertices) { index(vertices); }

            void index(const std::vector<VertexData> &vertices);
            bool empty() const {
                return indices.empty();
            }
            void clear() {
                indices.clear();
                vertices.clear();
                materials.clear();
                normals.clear();
            }
        };

        ShortIndexed(GLenum shape, const typename std::vector<VertexData> &vertices);
        ShortIndexed(GLenum shape, const typename std::vector<VertexData> &vertices, const typename std::vector<GLushort> &indices);
        ShortIndexed(GLenum shape, const typename ShortIndexed::Data &data);
        virtual ~ShortIndexed();

        uint draw(params params) override;

    private:
        void enableAllAttribs();
        void disableAllAttribs();
        void enableIndex();

        GLuint IndexBufferID;
        GLushort IndexSize = 0;

        GLuint MaterialBufferID;
        GLuint NormalBufferID;

        void setData(const ShortIndexed::Data &data);

        void setIndicies(const unsigned long size, const void *data);
        void setMaterials(const unsigned long size, const void *data);
        void setNormals(const unsigned long size, const void *data);
    };
}

