#include <vector>
#include <map>

#include <glm/glm.hpp>

#include "vboindexer.hpp"

bool getSimilarVertexIndex_fast(
	const buffer::VertexData &packed,
	std::map<buffer::VertexData, GLushort> &VertexToOutIndex,
	GLushort &result)
{
	std::map<buffer::VertexData, GLushort>::iterator it = VertexToOutIndex.find(packed);
	if ( it == VertexToOutIndex.end() ){
		return false;
	}else{
		result = it->second;
		return true;
	}
}
bool getSimilarVertexIndex_fast(
	const buffer::VertexData &packed,
	std::map<buffer::VertexData, unsigned int> &VertexToOutIndex,
	unsigned int &result)
{
	std::map<buffer::VertexData, unsigned int>::iterator it = VertexToOutIndex.find(packed);
	if ( it == VertexToOutIndex.end() ){
		return false;
	}else{
		result = it->second;
		return true;
	}
}


void indexVBO(
	std::vector<glm::vec3> &in_vertices,
	std::vector<GLushort> &in_materials,
	std::vector<glm::vec3> &in_normals,

	std::vector<unsigned int> &out_indices,
	std::vector<glm::vec3> &out_vertices,
	std::vector<GLushort> &out_materials,
	std::vector<glm::vec3> &out_normals)
{
	std::map<buffer::VertexData,unsigned int> VertexToOutIndex;

	out_indices.reserve(in_vertices.size());
	// For each input vertex
	for (unsigned int i = 0; i < in_vertices.size(); i++) {

		buffer::VertexData packed = {in_vertices[i], in_materials[i], in_normals[i]};

		// Try to find a similar vertex in out_XXXX
		unsigned int index;
		bool found = getSimilarVertexIndex_fast( packed, VertexToOutIndex, index);

		if ( found ){ // A similar vertex is already in the VBO, use it instead !
			out_indices.push_back( index );
		}else{ // If not, it needs to be added in the output data.
			out_vertices.push_back( in_vertices[i]);
			out_materials.push_back( in_materials[i]);
			out_normals.push_back( in_normals[i]);
			unsigned int newindex = (unsigned int)out_vertices.size() - 1;
			out_indices.push_back( newindex );
			VertexToOutIndex[ packed ] = newindex;
		}
	}
}
void indexVBO(
	const std::vector<buffer::VertexData> &in_vertices,

	std::vector<GLushort> &out_indices,
	std::vector<glm::vec3> &out_vertices,
	std::vector<GLushort> &out_materials,
	std::vector<glm::vec3> &out_normals)
{
	std::map<buffer::VertexData, GLushort> VertexToOutIndex;

	out_indices.reserve(in_vertices.size());
	// For each input vertex
	for (size_t i=0; i<in_vertices.size(); i++ ){
		// Try to find a similar vertex in out_XXXX
		GLushort index;
		bool found = getSimilarVertexIndex_fast(in_vertices[i], VertexToOutIndex, index);

		if ( found ){ // A similar vertex is already in the VBO, use it instead !
			out_indices.push_back( index );
		}else{ // If not, it needs to be added in the output data.
			out_vertices.push_back( in_vertices[i].Position);
			out_materials.push_back( in_vertices[i].Material);
			out_normals.push_back( in_vertices[i].Normal);
			GLushort newindex = (GLushort)out_vertices.size() - 1;
			out_indices.push_back( newindex );
			VertexToOutIndex[ in_vertices[i] ] = newindex;
		}
	}
}