#pragma once

#include "Program.hpp"

namespace pass {
    /// World voxels pass
    class MainProgram: public Program {
    public:
        /// Pass options
        struct options {
            /// Apply light properties
            bool pbr = true;
            /// Triplanar texture mapping
            bool triplanar = false;
            /// Active geometry pass
            bool geometry = true;
            /// Blend voxel with mixed materials (requires geometry)
            bool blend = false;
            /// Depth fog
            bool fog = true;
        };

        MainProgram(const options &opts);
        ~MainProgram();

        std::string getName() const override;
        void start(Renderer *) override;
        buffer::params setup(Renderer *, glm::mat4 modelMatrix) override;

        void setMVP(const GLfloat *matrix);
        void setModel(const GLfloat *matrix);
        void setView(const GLfloat *matrix);

        void bindTexture(const GLuint textureID);
        void bindNormal(const GLuint textureID);
        void bindHOS(const GLuint textureID);

        void setLightInvDir(const GLfloat *pos);
        void setFog(const GLfloat *color, GLfloat depth);

    private:
        GLuint MVPMatrixID;
        GLuint ModelMatrixID;
        GLuint ViewMatrixID;

        GLuint TextureID;
        GLuint NormalID;
        GLuint HOSID;

        GLuint LightInvDirID;
        GLuint FogDepthID;
        GLuint FogColorID;
    };
}