#pragma once

#include <map>

namespace world {
    /// Universe unit
    struct Voxel {
        /// Packed value
        /// swap(1) + material(12) + density(3)
        uint16_t value;
        using material_t = uint_fast16_t;
        using density_t = uint_fast8_t;
        constexpr static const density_t DENSITY_MAX = 0b0111;

        Voxel(uint16_t value = 0): value(value) { }
        Voxel(material_t material, density_t density, bool swap = false) {
            assert(density <= DENSITY_MAX);
            assert(material < (1 << 12));
            value = (swap & 0b1000'0000'0000'0000) |
                    ((material << 3) & 0b0111'1111'1111'1000) |
                    (density & DENSITY_MAX);
        }

        /// Quantity of material
        constexpr inline density_t density() const {
            return value & DENSITY_MAX;
        }
        /// Material type
        constexpr inline material_t material() const {
            return (value & 0b0111'1111'1111'1000) >> 3;
        }
        /// Swap value
        /// Use external metadata table
        constexpr inline bool swap() const {
            return value & 0b1000'0000'0000'0000;
        }
    };
    /// Stock of material
    struct Item {
        /// Quantity of material
        unsigned long long Count;
        /// Material type
        /// @see world::materials
        Voxel::material_t Material;
    };
    /// List of materials
    struct ItemList: std::map<unsigned short, unsigned long long> {
        void add(const std::optional<Item>& item) {
            if(item) {
                (*this)[item.value().Material] += item.value().Count;
            }
        }
    };
}
