#pragma once

#include <vector>
#include <string>
#include <shared_mutex>
#include "../forward.h"
#include "common.hpp"
#include "../../data/math.hpp"

namespace world {
    ///Group of chunks saved as a single file only pointer
    class FileRegion {
    public:
        FileRegion(const std::string& folderPath, const area_<region_pos> &pos);
        ~FileRegion();

        typedef std::vector<char> data;

        bool read(const region_chunk_pos &pos, const read_ctx& ctx, data &out);
        void write(const region_chunk_pos &pos, const write_ctx& ctx, const std::string_view &in);

    private:
        void save(std::optional<std::pair<region_chunk_pos, std::unique_ptr<FileRegion::data>>> added);

        std::string path;
        //TODO: use tickets to remove unused regions

        std::shared_mutex mutex;
        std::ifstream file;
        robin_hood::unordered_map<region_chunk_pos, std::pair<ushort, std::streampos>> index;

        void load();
    };
}